/*
 * SonarLint for IntelliJ IDEA
 * Copyright (C) 2015-2022 SonarSource
 * sonarlint@sonarsource.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonarlint.intellij.issue.vulnerabilities

import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.ProjectRootManager
import com.intellij.openapi.vfs.VirtualFile
import org.sonarlint.intellij.issue.Flow
import org.sonarlint.intellij.issue.IssueMatcher
import org.sonarlint.intellij.issue.IssueMatcher.NoMatchException
import org.sonarlint.intellij.issue.Location
import org.sonarlint.intellij.issue.fileOnlyLocation
import org.sonarlint.intellij.issue.resolvedLocation
import org.sonarlint.intellij.issue.unknownLocation
import org.sonarsource.sonarlint.core.client.api.connected.ServerIssue
import org.sonarsource.sonarlint.core.client.api.connected.ServerIssueLocation

class TaintVulnerabilityMatcher(private val project: Project) {
  private val issueMatcher = IssueMatcher(project)

  fun match(remoteIssue: ServerIssue): LocalTaintVulnerability {
    return LocalTaintVulnerability(
      matchLocation(remoteIssue),
      remoteIssue.flows.map { f -> Flow(f.locations().reversed().map { matchLocation(it) }) },
      remoteIssue)
  }

  private fun matchLocation(location: ServerIssueLocation): Location {
    for (contentRoot in ProjectRootManager.getInstance(project)
      .contentRoots) {
      if (contentRoot.isDirectory()) {
        val matchedFile = contentRoot.findFileByRelativePath(location.filePath!!)
        if (matchedFile != null) {
          return matchTextRange(matchedFile, location)
        }
      } else {
        // On Rider, all source files are returned as individual content roots, so simply check for equality
        if (contentRoot.path.endsWith(location.filePath!!)) {
          return matchTextRange(contentRoot, location)
        }
      }
    }
    return unknownLocation(location.message, location.filePath)
  }

  private fun matchTextRange(matchedFile: VirtualFile, location: ServerIssueLocation): Location {
    val textRange = location.textRange
    val message = location.message
    if (textRange == null) {
      return fileOnlyLocation(matchedFile, message)
    }
    return try {
      val rangeMarker = issueMatcher.match(matchedFile, textRange)
      resolvedLocation(matchedFile, rangeMarker, message, location.codeSnippet)
    } catch (e: NoMatchException) {
      fileOnlyLocation(matchedFile, message)
    }
  }
}
